% ====================================================
% Matlab function for transforming RGGB rawimage in 
% Matlab GRBG.
%
% Copyright (C) 2000-2001 by K-Team, S.A. All rights reserved.
%
% Developed at K-Team S.A by Skye Legon.
%
% Permission to use, copy, modify, and distribute this
% software is freely granted, provided that this notice
% is preserved.
%
% There is no warranty for this free software.  If the software is modified
% by someone else and passed on, we want its recipients to know that
% what they have is not the original, so that any problems introduced
% by others will not reflect on the original authors' reputations.
%
% ====================================================

function colorize=fun(snap)

snap = fliplr(snap/256);

% convert to colour

disp('Converting to colour...')

red = zeros(size(snap));
green = zeros(size(snap));
blue = zeros(size(snap));

xdim = size(snap,2);
ydim = size(snap,1);

% exchange vertical columns
%newimage = zeros(size(snap));
%for i=1:xdim
%	if mod(i,2)==1 % odd
%		newimage(:,i) = snap(:,i+1);
%	else
%		newimage(:,i) = snap(:,i-1);
%	end
%end
%snap = newimage;

disp('Transforming corners...')

% corner conditions
%  top left 
%  green (blue vertical, red lateral)
r=1; c=1;
red(r,c) = snap(r,c+1);
green(r,c) = snap(r,c);
blue(r,c) = snap(r+1,c);
%  top right
%  red
r=1; c=xdim;
red(r,c) = snap(r,c);
green(r,c) = (snap(r,c-1) + snap(r+1,c))/2;
blue(r,c) = snap(r+1,c-1);
%  bottom left
%  blue
r=ydim; c=1;
red(r,c) = snap(r-1,c+1);
green(r,c) = (snap(r-1,c) + snap(r,c+1))/2;
blue(r,c) = snap(r,c);
%  bottom right
%  green (red vertical, blue lateral)
r=ydim; c=xdim;
red(r,c) = snap(r-1,c);
green(r,c) = snap(r,c);
blue(r,c) = snap(r,c-1);

% edge conditions
disp('Transforming edges...')

for c=2:xdim-1
	% top row
	r = 1;
	if mod(c,2)==1 % odd
		%  green (blue vertical, red lateral)
		red(r,c) = (snap(r,c-1) + snap(r,c+1))/2;
		green(r,c) = snap(r,c);
		blue(r,c) = snap(r+1,c);
	else
		%  red
		red(r,c) = snap(r,c);
		green(r,c) = (snap(r,c-1) + snap(r,c+1))/2;
		blue(r,c) = (snap(r+1,c-1) + snap(r+1,c+1))/2;
	end
	% bottom row
	r = ydim;
	if mod(c,2)==1 % odd
		%  blue
		red(r,c) = (snap(r-1,c-1) + snap(r-1,c+1))/2;
		green(r,c) = snap(r-1,c);
		blue(r,c) = snap(r,c);
	else
		%  green (red vertical, blue lateral)
		red(r,c) = snap(r-1,c);
		green(r,c) = snap(r,c);
		blue(r,c) = (snap(r,c-1) + snap(r,c+1))/2;
	end
end

for r=2:ydim-1
	% left column
	c = 1;
	if mod(r,2)==1 % odd
		%  green (blue vertical, red lateral)
		red(r,c) = snap(r,c+1);
		green(r,c) = snap(r,c);
		blue(r,c) = (snap(r-1,c) + snap(r+1,c))/2;
	else
		%  blue
		red(r,c) = (snap(r-1,c+1) + snap(r+1,c+1))/2;
		green(r,c) = (snap(r-1,c) + snap(r+1,c) + snap(r,c+1))/3;
		blue(r,c) = snap(r,c);
	end
	% right column
	c = xdim;
	if mod(r,2)==1 % odd
		red(r,c) = snap(r,c);
		green(r,c) = (snap(r-1,c) + snap(r+1,c) + snap(r,c-1))/3;
		blue(r,c) = (snap(r-1,c-1) + snap(r+1,c-1))/2;
	else
		%  green (red vertical, blue lateral)
		red(r,c) = (snap(r-1,c) + snap(r+1,c))/2;
		green(r,c) = snap(r,c);
		blue(r,c) = snap(r,c-1);
	end
end

% centre of image (note it is GBRB)
disp('Transforming centre pixels...')
for c=2:xdim-1,
	for r=2:ydim-1,
		if mod(r,2)==1 % odd
			if mod(c,2)==1 % odd
				% green (blue vertical, red lateral)
				red(r,c) = (snap(r,c-1) + snap(r,c+1))/2;
				green(r,c) = snap(r,c);
				blue(r,c) = (snap(r-1,c) + snap(r+1,c))/2;
			else
				% red
				red(r,c) = snap(r,c);
				green(r,c) = (snap(r,c-1)+snap(r,c+1)+snap(r-1,c)+snap(r+1,c))/4;
				blue(r,c) = (snap(r-1,c-1)+snap(r-1,c+1)+snap(r+1,c-1)+snap(r+1,c+1))/4;
			end
		else
			if mod(c,2)==1 % odd
				% blue
				red(r,c) = (snap(r-1,c-1)+snap(r-1,c+1)+snap(r+1,c-1)+snap(r+1,c+1))/4;
				green(r,c) = (snap(r,c-1)+snap(r,c+1)+snap(r-1,c)+snap(r+1,c))/4;
				blue(r,c) = snap(r,c);
			else
				% green (red vertical, blue lateral)
				red(r,c) = (snap(r-1,c) + snap(r+1,c))/2;
				green(r,c) = snap(r,c);
				blue(r,c) = (snap(r,c-1) + snap(r,c+1))/2;
			end
		end
	end
end

newimage = fliplr(red);
newimage(:,:,2) = fliplr(green);
newimage(:,:,3) = fliplr(blue);

colorize = newimage;
