#define DEBUG 0

/*
 * EthnoExpo camera software
 *
 * Skye Legon
 * copyright K-Team SA 2000
 */

/* Include libraries */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

/* Include preprocessor defines */

#include "general.def"
#define EXPOSURE 50
#define THRESHOLD 250
#define MAX_CLUSTERS 30
#define PIXEL_DIST 20
#define XDIM 160
#define YDIM 120

/* flag pointers */
#define GRAB_NOW		0
#define GRAB_DONE		1
#define NUM_POINTS 	2
#define COORDS 			3

/* Declare globals (all globals begin with an underscore "_" character) */

/* process declarations */
uint32	_pidMainProcess = 0;
static char _nameMainProcess[] 	= "procMainProcess\r\n";

void procMainProcess()
{
	uint8 *flags = vvl_get_flag_ptr();
	uint8 *image = vvl_get_image_ptr();
	int32 receivedByte;
	/* store all points from indices 1 to n+1, not 0 to n */
	uint8 xproto[MAX_CLUSTERS+1], yproto[MAX_CLUSTERS+1];
	uint16 xmean[MAX_CLUSTERS+1], ymean[MAX_CLUSTERS+1]; 
	uint16 points[MAX_CLUSTERS+1];
	boolean clusterOverflow = FALSE;
	uint8 i,j,cluster,clusters=0;
	#if DEBUG
	uint8 yMax, maxVal;
	#endif

	/* watch flag to get command from Kameleon */
	for (EVER)
	{
		printf("Waiting for command from Kameleon or 'g' from user ('x' to exit to SERCOM)...\r\n\n");
		receivedByte = -1;
		while ((flags[GRAB_NOW]==0) && 
					((char)receivedByte!='g') && 
					((char)receivedByte!='x')) 
		{
			//printf("Grab flag is %3d \n\r",flags[GRAB_NOW]);
			WAIT(100);
			receivedByte = com_receive_byte();
		}

		//if ((char)receivedByte=='x') bios_restart_system();
		if ((char)receivedByte=='x') bios_restart_launch('FUSC', ser_get_config());

		flags[GRAB_DONE] = FALSE; /* in case it is still true */

		var_on_led(1);
		/* perform grab */
		printf("Grabbing...\n\r");
		vvl_aqu_image();

		/* threshold */
		/* note we haven't averaged any colours or converted to greyscale.  
		It works fine anyway. */
		printf("Thresholding and Classifying...\n\r");
		clusters = 0;
		clusterOverflow = FALSE;
		for (i=0;i<XDIM;i++)
		{
			#if DEBUG
			yMax = 0;
			maxVal = 0;
			#endif
			for (j=0;j<YDIM;j++)
			{
				#if DEBUG
				if (image[i+j*XDIM] > maxVal)
				{
					maxVal = image[i+j*XDIM];
					yMax = j;
				}
				#endif
				/* check to see if pixel exceeds threshold */
				if ((image[i+j*XDIM] >= THRESHOLD) && (!clusterOverflow))
				{
					cluster = 0;
					/* loop through all previously found clusters to see if this pixel
					is close, and if so assign this pixel to the cluster.  Do until
					a cluster is found or there are no more clusters */
					while (cluster<clusters)
					{
						cluster++;
						/* use a sort of Manhattan distance */
						if ((abs(i-xproto[cluster]) < PIXEL_DIST) && 
								(abs(j-yproto[cluster]) < PIXEL_DIST))
						{
							/* set pixel value to cluster ID - this only works if we
							have fewer clusters than our threshold level!  As we expect 2
							clusters, and the threshold is 250, no problem.
							(we could create a separate "cluster array" but
							the vv doesn't have much RAM */
							image[i+j*XDIM] = cluster;
							#if DEBUG
							printf("Assigned point (%3d,%3d) to cluster %2d\n\r",i,j,cluster);
							#endif
							break;
						}
					}
					/* if no cluster was found for the pixel (i.e. pixel value is
					still > THRESHOLD), create a new one */
					if (image[i+j*XDIM] >= THRESHOLD)
					{
						if (cluster < MAX_CLUSTERS)
						{
							clusters++;
							printf("Found cluster %2d starting at point (%3d,%3d).\n\r",
								clusters,i,j);
							cluster = clusters;
							xproto[cluster] = i;
							yproto[cluster] = j;
							xmean[cluster] = 0;
							ymean[cluster] = 0;
							points[cluster] = 0;
							image[i+j*XDIM] = cluster;
						}
						else
						{
							printf("Over %d clusters found.  Ignoring the rest...\n\r",
								MAX_CLUSTERS);
							clusterOverflow = TRUE;
						}
					}
					/* now record pixel location if we didn't overflow */
					if (!clusterOverflow)
					{
						xmean[cluster] += i;
						ymean[cluster] += j;
						points[cluster]++;
					}
				}
				else 
					image[i+j*XDIM] = 0;
			}
			#if DEBUG
			printf("Column %3d max at (%3d,%3d) %3d \n\r",i,i,yMax,maxVal);
			#endif
		}

		/* now calculate the means */
		printf("Found %2d clusters.\n\r",clusters);
		for (cluster=1;cluster<=clusters;cluster++)
		{
			/* add 1 to each point, as the Matlab calibration uses a coord
			system 1 to 160, but here it is 0 to 159 */
			i = (uint8)(xmean[cluster]/points[cluster])+1;
			j = (uint8)(ymean[cluster]/points[cluster])+1;
			flags[COORDS+(cluster-1)*2] = i;
			flags[COORDS+(cluster-1)*2+1] = j;
			printf("Cluster %d is roughly centred at pixel (%d,%d) \n\r",
				cluster, i, j);
		}

		#if DEBUG
		printf("\n\rAfter thresholding: \n\r");
		for (i=0;i<XDIM;i++)
		{
			yMax = 0;
			maxVal = 0;
			for (j=0;j<YDIM;j++)
			{
				if (image[i+j*XDIM] > maxVal)
				{
					maxVal = image[i+j*XDIM];
					yMax = j;
				}
			}
			printf("Column %3d max at (%3d,%3d) %3d \n\r",i,i,yMax,maxVal);
		}
		#endif

		var_off_led(1);
		flags[NUM_POINTS] = clusters;
		/* set flag to zero to signal we're done */
		flags[GRAB_DONE] = TRUE;
		/* reset GRAB_NOW */
		flags[GRAB_NOW] = FALSE;
	}
}

int main()
{
	/* Initialization */

  com_reset();
  var_reset();
  vvl_reset();

  /* disable auto exposure and gain */
  vvl_write_reg(16,0);
  /* set coarse exposure to 100, fine to 0 */
  vvl_write_reg(35,EXPOSURE);
  vvl_write_reg(33,0);
  /* set gain to 1.00 */
  vvl_write_reg(36,0);

  _pidMainProcess = install_task(_nameMainProcess, 800, procMainProcess);
  if (_pidMainProcess == -1) exit(0);

  return 0;
}
